# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: device_msgs.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import common_msgs_pb2 as common__msgs__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='device_msgs.proto',
  package='Nrmk.IndyFramework',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x11\x64\x65vice_msgs.proto\x12\x12Nrmk.IndyFramework\x1a\x11\x63ommon_msgs.proto\"&\n\x05Motor\x12\r\n\x05index\x18\x01 \x01(\r\x12\x0e\n\x06\x65nable\x18\x02 \x01(\x08\"&\n\x05Servo\x12\r\n\x05index\x18\x01 \x01(\r\x12\x0e\n\x06\x65nable\x18\x02 \x01(\x08\"Q\n\rDigitalSignal\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\r\x12/\n\x05state\x18\x02 \x01(\x0e\x32 .Nrmk.IndyFramework.DigitalState\"O\n\rEndtoolSignal\x12\x0c\n\x04port\x18\x01 \x01(\t\x12\x30\n\x06states\x18\x02 \x03(\x0e\x32 .Nrmk.IndyFramework.EndtoolState\"0\n\x0c\x41nalogSignal\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\r\x12\x0f\n\x07voltage\x18\x02 \x01(\x05\"6\n\tMotorList\x12)\n\x06motors\x18\x01 \x03(\x0b\x32\x19.Nrmk.IndyFramework.Motor\"A\n\x0b\x44igitalList\x12\x32\n\x07signals\x18\x01 \x03(\x0b\x32!.Nrmk.IndyFramework.DigitalSignal\"G\n\x11\x45ndtoolSignalList\x12\x32\n\x07signals\x18\x01 \x03(\x0b\x32!.Nrmk.IndyFramework.EndtoolSignal\"?\n\nAnalogList\x12\x31\n\x07signals\x18\x01 \x03(\x0b\x32 .Nrmk.IndyFramework.AnalogSignal\"\x96\x02\n\nDeviceInfo\x12\x12\n\nnum_joints\x18\x01 \x01(\r\x12\x14\n\x0crobot_serial\x18\x02 \x01(\t\x12\x0f\n\x07payload\x18\x03 \x01(\x02\x12\x17\n\x0fio_board_fw_ver\x18\n \x01(\t\x12\x1a\n\x12\x63ore_board_fw_vers\x18\x0b \x03(\t\x12\x1c\n\x14\x65ndtool_board_fw_ver\x18\x0c \x01(\t\x12\x16\n\x0e\x63ontroller_ver\x18\x14 \x01(\t\x12\x19\n\x11\x63ontroller_detail\x18\x15 \x01(\t\x12\x17\n\x0f\x63ontroller_date\x18\x16 \x01(\t\x12.\n\x08response\x18\x64 \x01(\x0b\x32\x1c.Nrmk.IndyFramework.Response\"\xc7\x01\n\x11\x42rakeControlStyle\x12\x41\n\x05style\x18\x01 \x01(\x0e\x32\x32.Nrmk.IndyFramework.BrakeControlStyle.ControlStyle\x12.\n\x08response\x18\x64 \x01(\x0b\x32\x1c.Nrmk.IndyFramework.Response\"?\n\x0c\x43ontrolStyle\x12\x0f\n\x0bUNAVAILABLE\x10\x00\x12\x0e\n\nCONCURRENT\x10\x01\x12\x0e\n\nINDIVIDUAL\x10\x02\"~\n\rGetEL5001Data\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x05\x12\r\n\x05\x64\x65lta\x18\x03 \x01(\x05\x12\x0f\n\x07\x61verage\x18\x04 \x01(\x02\x12.\n\x08response\x18\x64 \x01(\x0b\x32\x1c.Nrmk.IndyFramework.Response\"\x8d\x01\n\rGetEL5101Data\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x05\x12\r\n\x05latch\x18\x03 \x01(\x05\x12\r\n\x05\x64\x65lta\x18\x04 \x01(\x05\x12\x0f\n\x07\x61verage\x18\x05 \x01(\x02\x12.\n\x08response\x18\x64 \x01(\x0b\x32\x1c.Nrmk.IndyFramework.Response\"V\n\x0c\x46TSensorType\x12\x16\n\x0e\x66t_sensor_type\x18\x01 \x01(\t\x12.\n\x08response\x18\x64 \x01(\x0b\x32\x1c.Nrmk.IndyFramework.Response\"\x98\x01\n\x0c\x46TSensorData\x12\r\n\x05\x66t_Fx\x18\x01 \x01(\x02\x12\r\n\x05\x66t_Fy\x18\x02 \x01(\x02\x12\r\n\x05\x66t_Fz\x18\x03 \x01(\x02\x12\r\n\x05\x66t_Tx\x18\x04 \x01(\x02\x12\r\n\x05\x66t_Ty\x18\x05 \x01(\x02\x12\r\n\x05\x66t_Tz\x18\x06 \x01(\x02\x12.\n\x08response\x18\x64 \x01(\x0b\x32\x1c.Nrmk.IndyFramework.Response*=\n\x0c\x44igitalState\x12\r\n\tOFF_STATE\x10\x00\x12\x0c\n\x08ON_STATE\x10\x01\x12\x10\n\x0cUNUSED_STATE\x10\x02*b\n\x0c\x45ndtoolState\x12\n\n\x06UNUSED\x10\x00\x12\x0c\n\x08HIGH_PNP\x10\x02\x12\x0c\n\x08HIGH_NPN\x10\x01\x12\x14\n\x07LOW_NPN\x10\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01\x12\x14\n\x07LOW_PNP\x10\xfe\xff\xff\xff\xff\xff\xff\xff\xff\x01\x62\x06proto3'
  ,
  dependencies=[common__msgs__pb2.DESCRIPTOR,])

_DIGITALSTATE = _descriptor.EnumDescriptor(
  name='DigitalState',
  full_name='Nrmk.IndyFramework.DigitalState',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OFF_STATE', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ON_STATE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNUSED_STATE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1613,
  serialized_end=1674,
)
_sym_db.RegisterEnumDescriptor(_DIGITALSTATE)

DigitalState = enum_type_wrapper.EnumTypeWrapper(_DIGITALSTATE)
_ENDTOOLSTATE = _descriptor.EnumDescriptor(
  name='EndtoolState',
  full_name='Nrmk.IndyFramework.EndtoolState',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNUSED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='HIGH_PNP', index=1, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='HIGH_NPN', index=2, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOW_NPN', index=3, number=-1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LOW_PNP', index=4, number=-2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1676,
  serialized_end=1774,
)
_sym_db.RegisterEnumDescriptor(_ENDTOOLSTATE)

EndtoolState = enum_type_wrapper.EnumTypeWrapper(_ENDTOOLSTATE)
OFF_STATE = 0
ON_STATE = 1
UNUSED_STATE = 2
UNUSED = 0
HIGH_PNP = 2
HIGH_NPN = 1
LOW_NPN = -1
LOW_PNP = -2


_BRAKECONTROLSTYLE_CONTROLSTYLE = _descriptor.EnumDescriptor(
  name='ControlStyle',
  full_name='Nrmk.IndyFramework.BrakeControlStyle.ControlStyle',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNAVAILABLE', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CONCURRENT', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INDIVIDUAL', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1033,
  serialized_end=1096,
)
_sym_db.RegisterEnumDescriptor(_BRAKECONTROLSTYLE_CONTROLSTYLE)


_MOTOR = _descriptor.Descriptor(
  name='Motor',
  full_name='Nrmk.IndyFramework.Motor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='index', full_name='Nrmk.IndyFramework.Motor.index', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='enable', full_name='Nrmk.IndyFramework.Motor.enable', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=60,
  serialized_end=98,
)


_SERVO = _descriptor.Descriptor(
  name='Servo',
  full_name='Nrmk.IndyFramework.Servo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='index', full_name='Nrmk.IndyFramework.Servo.index', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='enable', full_name='Nrmk.IndyFramework.Servo.enable', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=100,
  serialized_end=138,
)


_DIGITALSIGNAL = _descriptor.Descriptor(
  name='DigitalSignal',
  full_name='Nrmk.IndyFramework.DigitalSignal',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='address', full_name='Nrmk.IndyFramework.DigitalSignal.address', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='state', full_name='Nrmk.IndyFramework.DigitalSignal.state', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=140,
  serialized_end=221,
)


_ENDTOOLSIGNAL = _descriptor.Descriptor(
  name='EndtoolSignal',
  full_name='Nrmk.IndyFramework.EndtoolSignal',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='port', full_name='Nrmk.IndyFramework.EndtoolSignal.port', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='states', full_name='Nrmk.IndyFramework.EndtoolSignal.states', index=1,
      number=2, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=223,
  serialized_end=302,
)


_ANALOGSIGNAL = _descriptor.Descriptor(
  name='AnalogSignal',
  full_name='Nrmk.IndyFramework.AnalogSignal',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='address', full_name='Nrmk.IndyFramework.AnalogSignal.address', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='voltage', full_name='Nrmk.IndyFramework.AnalogSignal.voltage', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=304,
  serialized_end=352,
)


_MOTORLIST = _descriptor.Descriptor(
  name='MotorList',
  full_name='Nrmk.IndyFramework.MotorList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='motors', full_name='Nrmk.IndyFramework.MotorList.motors', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=354,
  serialized_end=408,
)


_DIGITALLIST = _descriptor.Descriptor(
  name='DigitalList',
  full_name='Nrmk.IndyFramework.DigitalList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='signals', full_name='Nrmk.IndyFramework.DigitalList.signals', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=410,
  serialized_end=475,
)


_ENDTOOLSIGNALLIST = _descriptor.Descriptor(
  name='EndtoolSignalList',
  full_name='Nrmk.IndyFramework.EndtoolSignalList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='signals', full_name='Nrmk.IndyFramework.EndtoolSignalList.signals', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=477,
  serialized_end=548,
)


_ANALOGLIST = _descriptor.Descriptor(
  name='AnalogList',
  full_name='Nrmk.IndyFramework.AnalogList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='signals', full_name='Nrmk.IndyFramework.AnalogList.signals', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=550,
  serialized_end=613,
)


_DEVICEINFO = _descriptor.Descriptor(
  name='DeviceInfo',
  full_name='Nrmk.IndyFramework.DeviceInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='num_joints', full_name='Nrmk.IndyFramework.DeviceInfo.num_joints', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='robot_serial', full_name='Nrmk.IndyFramework.DeviceInfo.robot_serial', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='Nrmk.IndyFramework.DeviceInfo.payload', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='io_board_fw_ver', full_name='Nrmk.IndyFramework.DeviceInfo.io_board_fw_ver', index=3,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='core_board_fw_vers', full_name='Nrmk.IndyFramework.DeviceInfo.core_board_fw_vers', index=4,
      number=11, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='endtool_board_fw_ver', full_name='Nrmk.IndyFramework.DeviceInfo.endtool_board_fw_ver', index=5,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='controller_ver', full_name='Nrmk.IndyFramework.DeviceInfo.controller_ver', index=6,
      number=20, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='controller_detail', full_name='Nrmk.IndyFramework.DeviceInfo.controller_detail', index=7,
      number=21, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='controller_date', full_name='Nrmk.IndyFramework.DeviceInfo.controller_date', index=8,
      number=22, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='response', full_name='Nrmk.IndyFramework.DeviceInfo.response', index=9,
      number=100, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=616,
  serialized_end=894,
)


_BRAKECONTROLSTYLE = _descriptor.Descriptor(
  name='BrakeControlStyle',
  full_name='Nrmk.IndyFramework.BrakeControlStyle',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='style', full_name='Nrmk.IndyFramework.BrakeControlStyle.style', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='response', full_name='Nrmk.IndyFramework.BrakeControlStyle.response', index=1,
      number=100, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _BRAKECONTROLSTYLE_CONTROLSTYLE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=897,
  serialized_end=1096,
)


_GETEL5001DATA = _descriptor.Descriptor(
  name='GetEL5001Data',
  full_name='Nrmk.IndyFramework.GetEL5001Data',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='Nrmk.IndyFramework.GetEL5001Data.status', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='Nrmk.IndyFramework.GetEL5001Data.value', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='delta', full_name='Nrmk.IndyFramework.GetEL5001Data.delta', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='average', full_name='Nrmk.IndyFramework.GetEL5001Data.average', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='response', full_name='Nrmk.IndyFramework.GetEL5001Data.response', index=4,
      number=100, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1098,
  serialized_end=1224,
)


_GETEL5101DATA = _descriptor.Descriptor(
  name='GetEL5101Data',
  full_name='Nrmk.IndyFramework.GetEL5101Data',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='Nrmk.IndyFramework.GetEL5101Data.status', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='Nrmk.IndyFramework.GetEL5101Data.value', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='latch', full_name='Nrmk.IndyFramework.GetEL5101Data.latch', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='delta', full_name='Nrmk.IndyFramework.GetEL5101Data.delta', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='average', full_name='Nrmk.IndyFramework.GetEL5101Data.average', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='response', full_name='Nrmk.IndyFramework.GetEL5101Data.response', index=5,
      number=100, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1227,
  serialized_end=1368,
)


_FTSENSORTYPE = _descriptor.Descriptor(
  name='FTSensorType',
  full_name='Nrmk.IndyFramework.FTSensorType',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ft_sensor_type', full_name='Nrmk.IndyFramework.FTSensorType.ft_sensor_type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='response', full_name='Nrmk.IndyFramework.FTSensorType.response', index=1,
      number=100, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1370,
  serialized_end=1456,
)


_FTSENSORDATA = _descriptor.Descriptor(
  name='FTSensorData',
  full_name='Nrmk.IndyFramework.FTSensorData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ft_Fx', full_name='Nrmk.IndyFramework.FTSensorData.ft_Fx', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ft_Fy', full_name='Nrmk.IndyFramework.FTSensorData.ft_Fy', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ft_Fz', full_name='Nrmk.IndyFramework.FTSensorData.ft_Fz', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ft_Tx', full_name='Nrmk.IndyFramework.FTSensorData.ft_Tx', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ft_Ty', full_name='Nrmk.IndyFramework.FTSensorData.ft_Ty', index=4,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ft_Tz', full_name='Nrmk.IndyFramework.FTSensorData.ft_Tz', index=5,
      number=6, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='response', full_name='Nrmk.IndyFramework.FTSensorData.response', index=6,
      number=100, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1459,
  serialized_end=1611,
)

_DIGITALSIGNAL.fields_by_name['state'].enum_type = _DIGITALSTATE
_ENDTOOLSIGNAL.fields_by_name['states'].enum_type = _ENDTOOLSTATE
_MOTORLIST.fields_by_name['motors'].message_type = _MOTOR
_DIGITALLIST.fields_by_name['signals'].message_type = _DIGITALSIGNAL
_ENDTOOLSIGNALLIST.fields_by_name['signals'].message_type = _ENDTOOLSIGNAL
_ANALOGLIST.fields_by_name['signals'].message_type = _ANALOGSIGNAL
_DEVICEINFO.fields_by_name['response'].message_type = common__msgs__pb2._RESPONSE
_BRAKECONTROLSTYLE.fields_by_name['style'].enum_type = _BRAKECONTROLSTYLE_CONTROLSTYLE
_BRAKECONTROLSTYLE.fields_by_name['response'].message_type = common__msgs__pb2._RESPONSE
_BRAKECONTROLSTYLE_CONTROLSTYLE.containing_type = _BRAKECONTROLSTYLE
_GETEL5001DATA.fields_by_name['response'].message_type = common__msgs__pb2._RESPONSE
_GETEL5101DATA.fields_by_name['response'].message_type = common__msgs__pb2._RESPONSE
_FTSENSORTYPE.fields_by_name['response'].message_type = common__msgs__pb2._RESPONSE
_FTSENSORDATA.fields_by_name['response'].message_type = common__msgs__pb2._RESPONSE
DESCRIPTOR.message_types_by_name['Motor'] = _MOTOR
DESCRIPTOR.message_types_by_name['Servo'] = _SERVO
DESCRIPTOR.message_types_by_name['DigitalSignal'] = _DIGITALSIGNAL
DESCRIPTOR.message_types_by_name['EndtoolSignal'] = _ENDTOOLSIGNAL
DESCRIPTOR.message_types_by_name['AnalogSignal'] = _ANALOGSIGNAL
DESCRIPTOR.message_types_by_name['MotorList'] = _MOTORLIST
DESCRIPTOR.message_types_by_name['DigitalList'] = _DIGITALLIST
DESCRIPTOR.message_types_by_name['EndtoolSignalList'] = _ENDTOOLSIGNALLIST
DESCRIPTOR.message_types_by_name['AnalogList'] = _ANALOGLIST
DESCRIPTOR.message_types_by_name['DeviceInfo'] = _DEVICEINFO
DESCRIPTOR.message_types_by_name['BrakeControlStyle'] = _BRAKECONTROLSTYLE
DESCRIPTOR.message_types_by_name['GetEL5001Data'] = _GETEL5001DATA
DESCRIPTOR.message_types_by_name['GetEL5101Data'] = _GETEL5101DATA
DESCRIPTOR.message_types_by_name['FTSensorType'] = _FTSENSORTYPE
DESCRIPTOR.message_types_by_name['FTSensorData'] = _FTSENSORDATA
DESCRIPTOR.enum_types_by_name['DigitalState'] = _DIGITALSTATE
DESCRIPTOR.enum_types_by_name['EndtoolState'] = _ENDTOOLSTATE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Motor = _reflection.GeneratedProtocolMessageType('Motor', (_message.Message,), {
  'DESCRIPTOR' : _MOTOR,
  '__module__' : 'device_msgs_pb2'
  # @@protoc_insertion_point(class_scope:Nrmk.IndyFramework.Motor)
  })
_sym_db.RegisterMessage(Motor)

Servo = _reflection.GeneratedProtocolMessageType('Servo', (_message.Message,), {
  'DESCRIPTOR' : _SERVO,
  '__module__' : 'device_msgs_pb2'
  # @@protoc_insertion_point(class_scope:Nrmk.IndyFramework.Servo)
  })
_sym_db.RegisterMessage(Servo)

DigitalSignal = _reflection.GeneratedProtocolMessageType('DigitalSignal', (_message.Message,), {
  'DESCRIPTOR' : _DIGITALSIGNAL,
  '__module__' : 'device_msgs_pb2'
  # @@protoc_insertion_point(class_scope:Nrmk.IndyFramework.DigitalSignal)
  })
_sym_db.RegisterMessage(DigitalSignal)

EndtoolSignal = _reflection.GeneratedProtocolMessageType('EndtoolSignal', (_message.Message,), {
  'DESCRIPTOR' : _ENDTOOLSIGNAL,
  '__module__' : 'device_msgs_pb2'
  # @@protoc_insertion_point(class_scope:Nrmk.IndyFramework.EndtoolSignal)
  })
_sym_db.RegisterMessage(EndtoolSignal)

AnalogSignal = _reflection.GeneratedProtocolMessageType('AnalogSignal', (_message.Message,), {
  'DESCRIPTOR' : _ANALOGSIGNAL,
  '__module__' : 'device_msgs_pb2'
  # @@protoc_insertion_point(class_scope:Nrmk.IndyFramework.AnalogSignal)
  })
_sym_db.RegisterMessage(AnalogSignal)

MotorList = _reflection.GeneratedProtocolMessageType('MotorList', (_message.Message,), {
  'DESCRIPTOR' : _MOTORLIST,
  '__module__' : 'device_msgs_pb2'
  # @@protoc_insertion_point(class_scope:Nrmk.IndyFramework.MotorList)
  })
_sym_db.RegisterMessage(MotorList)

DigitalList = _reflection.GeneratedProtocolMessageType('DigitalList', (_message.Message,), {
  'DESCRIPTOR' : _DIGITALLIST,
  '__module__' : 'device_msgs_pb2'
  # @@protoc_insertion_point(class_scope:Nrmk.IndyFramework.DigitalList)
  })
_sym_db.RegisterMessage(DigitalList)

EndtoolSignalList = _reflection.GeneratedProtocolMessageType('EndtoolSignalList', (_message.Message,), {
  'DESCRIPTOR' : _ENDTOOLSIGNALLIST,
  '__module__' : 'device_msgs_pb2'
  # @@protoc_insertion_point(class_scope:Nrmk.IndyFramework.EndtoolSignalList)
  })
_sym_db.RegisterMessage(EndtoolSignalList)

AnalogList = _reflection.GeneratedProtocolMessageType('AnalogList', (_message.Message,), {
  'DESCRIPTOR' : _ANALOGLIST,
  '__module__' : 'device_msgs_pb2'
  # @@protoc_insertion_point(class_scope:Nrmk.IndyFramework.AnalogList)
  })
_sym_db.RegisterMessage(AnalogList)

DeviceInfo = _reflection.GeneratedProtocolMessageType('DeviceInfo', (_message.Message,), {
  'DESCRIPTOR' : _DEVICEINFO,
  '__module__' : 'device_msgs_pb2'
  # @@protoc_insertion_point(class_scope:Nrmk.IndyFramework.DeviceInfo)
  })
_sym_db.RegisterMessage(DeviceInfo)

BrakeControlStyle = _reflection.GeneratedProtocolMessageType('BrakeControlStyle', (_message.Message,), {
  'DESCRIPTOR' : _BRAKECONTROLSTYLE,
  '__module__' : 'device_msgs_pb2'
  # @@protoc_insertion_point(class_scope:Nrmk.IndyFramework.BrakeControlStyle)
  })
_sym_db.RegisterMessage(BrakeControlStyle)

GetEL5001Data = _reflection.GeneratedProtocolMessageType('GetEL5001Data', (_message.Message,), {
  'DESCRIPTOR' : _GETEL5001DATA,
  '__module__' : 'device_msgs_pb2'
  # @@protoc_insertion_point(class_scope:Nrmk.IndyFramework.GetEL5001Data)
  })
_sym_db.RegisterMessage(GetEL5001Data)

GetEL5101Data = _reflection.GeneratedProtocolMessageType('GetEL5101Data', (_message.Message,), {
  'DESCRIPTOR' : _GETEL5101DATA,
  '__module__' : 'device_msgs_pb2'
  # @@protoc_insertion_point(class_scope:Nrmk.IndyFramework.GetEL5101Data)
  })
_sym_db.RegisterMessage(GetEL5101Data)

FTSensorType = _reflection.GeneratedProtocolMessageType('FTSensorType', (_message.Message,), {
  'DESCRIPTOR' : _FTSENSORTYPE,
  '__module__' : 'device_msgs_pb2'
  # @@protoc_insertion_point(class_scope:Nrmk.IndyFramework.FTSensorType)
  })
_sym_db.RegisterMessage(FTSensorType)

FTSensorData = _reflection.GeneratedProtocolMessageType('FTSensorData', (_message.Message,), {
  'DESCRIPTOR' : _FTSENSORDATA,
  '__module__' : 'device_msgs_pb2'
  # @@protoc_insertion_point(class_scope:Nrmk.IndyFramework.FTSensorData)
  })
_sym_db.RegisterMessage(FTSensorData)


# @@protoc_insertion_point(module_scope)



from neuromeka.proto import *
from neuromeka.common import *

import grpc, time


MOBY_PORT = 20200

class MobyClient:
    def __init__(self, ip_addr):
        moby_channel = grpc.insecure_channel("{}:{}".format(ip_addr, MOBY_PORT))
        moby_stub = MobyStub(moby_channel)

        self.__moby_stub = moby_stub

    @Utils.exception_handler
    def get_moby_state(self):
        """
        Get Moby State
        -State
        -is_ready
        -is_enable
        -is_moving
        -is_violation
        """
        state_dict = {
            0: "SYSTEM_OFF",
            1: "SYSTEM_ON",
            2: "VIOLATE",
            4: "RECOVER_SOFT",
            6: "IDLE",
            7: "MOVING",
            16: "TELE_OP",
        }
        state = self.__moby_stub.GetMobyState(Empty())
        moby_state = state_dict.get(state.status, "(UNKNOWN STATE CODE)")
        b_ready = state.is_ready
        b_enable = state.is_enable
        b_moving = state.is_moving
        b_violation = state.is_violation

        return {'state': moby_state, 'is_ready': b_ready,
                'is_enable': b_enable, 'is_moving': b_moving, 'is_violation': b_violation}

    @Utils.exception_handler
    def get_moby_error_state(self):
        """
        Get Moby Error State
        """
        error_dict = {
            0x00: "NONE",
            0x01: "SW_MASTER_NOT_OP",
            0x02: "SW_SLAVES_NOT_OP",
            0x04: "SW_SERVO_NOT_ON",
            0x08: "SW_SERVO_NOT_READY",
            0x10: "SW_ENCODER_ABNORMAL",
            0x20: "SW_BUMPER_DETECT",
            0x100: "HW_CONNECTION_LOST",
        }

        err = self.__moby_stub.GetMobyErrorState(Empty())

        return [error_dict.get(err.errorState, "(UNKNOWN ERROR CODE)"),
                err.errorIndex1, err.errorIndex2, err.errorIndex3]

    @Utils.exception_handler
    def recover(self):
        return self.__moby_stub.Recover(Empty())

    @Utils.exception_handler
    def get_moby_pose(self):
        """
        Get Moby pose (m): [Px, Py, Pw]
        """
        pose = self.__moby_stub.GetMobyPose(Empty())
        return [pose.px, pose.py, pose.pw]

    @Utils.exception_handler
    def get_moby_vel(self):
        """
        Get Moby velocity (m/s): [Vx, Vy, Vw]
        """
        vel = self.__moby_stub.GetMobyVel(Empty())
        return [vel.vx, vel.vy, vel.vw]

    @Utils.exception_handler
    def reset_moby_pose(self):
        """
        Reset Moby pose
        """
        return self.__moby_stub.ResetMobyPose(Empty())

    @Utils.exception_handler
    def get_rotation_angle(self):
        """
        Get rotation angle (deg): [fl, fr, bl, br]
        """
        val = self.__moby_stub.GetRotationAngleDeg(Empty())
        return {'fl': val.fl, 'fr': val.fr, 'bl':val.bl, 'br':val.br}

    @Utils.exception_handler
    def get_drive_speed(self):
        """
        Get drive speed (m/s): [fl, fr, bl, br]
        """
        val = self.__moby_stub.GetDriveSpeed(Empty())
        return {'fl': val.fl, 'fr': val.fr, 'bl':val.bl, 'br':val.br}

    @Utils.exception_handler
    def get_target_vel(self):
        """
        Get Moby's target velocity
        """
        target = self.__moby_stub.GetTargetVel(Empty())
        return [target.vx, target.vy, target.vw]

    @Utils.exception_handler
    def get_zero(self):
        """
        Get rotation's zero position (encoder count)
                                    [fl, fr, bl, br]
        """
        val = self.__moby_stub.GetRotationZeroCount(Empty())
        return {'fl': val.fl, 'fr': val.fr, 'bl':val.bl, 'br':val.br}

    @Utils.exception_handler
    def get_gyro_data(self):
        """
        Get Gyro sensor data (yaw, yaw rate)
        """
        return self.__moby_stub.GetGyroData(Empty()).val

    @Utils.exception_handler
    def get_imu_data(self):
        """
        Get Full IMU sensor data
        """
        data = self.__moby_stub.GetGyroFullData(Empty())
        angle = [data.angleX, data.angleY, data.angleZ]
        vel = [data.angleVelX, data.angleVelY, data.angleVelZ]
        acc = [data.linAccX, data.linAccY, data.linAccZ]
        return angle, vel, acc

    @Utils.exception_handler
    def reset_gyro(self):
        """
        Reset gyro sensor
        """
        return self.__moby_stub.ResetGyroSensor(Empty())

    @Utils.exception_handler
    def use_gyro_for_odom(self, use_gyro):
        """
        Use gyro sensor for odometry calculation
        """
        return self.__moby_stub.UseGyroForOdom(BoolVal(val=use_gyro))

    @Utils.exception_handler
    def get_us_data(self):
        """
        Get US sensor data
        """
        value = self.__moby_stub.GetUSSensorData(Empty())
        return {'front_left1': value.us_front_left1, 'front_left2': value.us_front_left2, 'front_left3': value.us_front_left3, 'front_ground': value.us_front_ground, 'front_right1': value.us_front_right1, 'front_right2': value.us_front_right2, 'front_right3': value.us_front_right3, 'front_right4': value.us_front_right4, 'back_right1': value.us_back_right1, 'back_right2': value.us_back_right2, 'back_right3': value.us_back_right3, 'back_ground': value.us_back_ground, 'back_left1': value.us_back_left1, 'back_left2': value.us_back_left2, 'back_left3': value.us_back_left3, 'back_left4': value.us_back_left4}
    

    @Utils.exception_handler
    def get_bms(self):
        """
        Get BMS data
        GreenPyzzle : 
            'BMS status-1', 'BMS status-2',
            'Pack voltage-1', 'Pack voltage-2',
            'Battery Voltage-1', 'Battery Voltage-2',
            'Pack current1-1', 'Pack current1-2', 'Pack current2-1', 'Pack current2-2',
            'Temperature-1', 'Temperature-2', 'Temperature-3', 'Temperature-4'
        CTNS : 
            'Pack voltage-1', 'Pack current1-1
            'Is Charge', 'Is Cell OverVoltage',
            'Is Cell UnderVoltage', 'Is OverCurrent Charge',
            'Is OverCurrent Discharge', 'Is Short Circuit',
            'Is OverTemperature', 'Is Pack OverVoltage',
            'SOC', 'SOH', 'Time for Charge', 'time for Discharge'
            'Remain Capacity Ah', 'Remain Capacity Wh'
            'Temperature-(1~3)', 'Cell Voltage-(1~13)'
        """
        value = self.__moby_stub.GetBMSData(Empty())
        return {'BMS status-1': value.bms_status[0] / 10, 'BMS status-2': value.bms_status[1] / 10,
                'Pack voltage-1': value.pack_volt[0] / 100, 'Pack voltage-2': value.pack_volt[1] / 100,
                'Battery Voltage-1': value.battery_volt[0] / 100, 'Battery Voltage-2': value.battery_volt[1] / 100,
                'Pack current1-1': value.pack_current1[0] / 100, 'Pack current1-2': value.pack_current1[1] / 100,
                'Pack current2-1': value.pack_current2[0] / 100, 'Pack current2-2': value.pack_current2[1] / 100,
                'Is Charge': value.isCharge, 'Is Cell OverVoltage': value.isCellOverVolt,
                'Is Cell UnderVoltage': value.isCellUnderVolt, 'Is OverCurrent Charge': value.isOverCurCharge,
                'Is OverCurrent Discharge': value.isOverCurDischrg, 'Is Short Circuit': value.isShortCircuit,
                'Is OverTemperature': value.isOverTemperature, 'Is Pack OverVoltage': value.isPackOverVolt,
                'SOC': value.SOC * 0.1, 'SOH': value.SOH, 'Time for Charge': value.time_charge,
                'time for Discharge': value.time_dcharge, 'Remain Capacity Ah': value.rem_capAh / 100,
                'Remain Capacity Wh': value.rem_capWh, 'Temperature-1': value.bms_temperature[0] * 0.1,
                'Temperature-2': value.bms_temperature[1] * 0.1, 'Temperature-3': value.bms_temperature[2] * 0.1,
                'Temperature-4': value.bms_temperature[3] * 0.1, 'Cell Voltage-1': value.cell_volt[0] * 0.001,
                'Cell Voltage-2': value.cell_volt[1] * 0.001, 'Cell Voltage-3': value.cell_volt[2] * 0.001,
                'Cell Voltage-4': value.cell_volt[3] * 0.001, 'Cell Voltage-5': value.cell_volt[4] * 0.001,
                'Cell Voltage-6': value.cell_volt[5] * 0.001, 'Cell Voltage-7': value.cell_volt[6] * 0.001,
                'Cell Voltage-8': value.cell_volt[7] * 0.001, 'Cell Voltage-9': value.cell_volt[8] * 0.001,
                'Cell Voltage-10': value.cell_volt[9] * 0.001, 'Cell Voltage-11': value.cell_volt[10] * 0.001,
                'Cell Voltage-12': value.cell_volt[11] * 0.001, 'Cell Voltage-13': value.cell_volt[12] * 0.001}

    @Utils.exception_handler
    def set_target_vel(self, vx, vy, vw):
        """
        Drive control
        """
        return self.__moby_stub.SetStepControl(TargetVel(vx=vx, vy=vy, vw=vw))

    @Utils.exception_handler
    def stop_motion(self):
        """
        Stop Moby motion
        """
        return self.__moby_stub.StopMotion(Empty())

    @Utils.exception_handler
    def go_straight(self):
        """
        Go straight (zero rotation)
        """
        return self.__moby_stub.SetRotationAngleDeg(SwerveDoubles(fl=0, fr=0, bl=0, br=0))

    @Utils.exception_handler
    def move_rotation_deg(self, fl, fr, bl, br):
        """
        Rotation control (target angle degree)
        """
        return self.__moby_stub.SetRotationAngleDeg(SwerveDoubles(fr=fr, br=br, bl=bl, fl=fl))

    @Utils.exception_handler
    def move_driving_mps(self, fl, fr, bl, br):
        """
        Driving control (target speed m/s)
        """
        return self.__moby_stub.DriveWheel(SwerveDoubles(fr=fr, br=br, bl=bl, fl=fl))

    ############################
    # Set Moby parameters
    ############################

    @Utils.exception_handler
    def set_zero_as_current(self):
        """
        Set current roation position as zero
        """
        return self.__moby_stub.SetZeroPosAsCurrentPos(Empty())

    @Utils.exception_handler
    def set_rotation_vel_acc(self, vel, acc):
        """
        Set rotation maximum velocity, acceleration
        """
        return self.__moby_stub.SetRotationVelAcc(DoubleVals(val=[vel, acc]))

    @Utils.exception_handler
    def set_rotation_interpolator(self, val):
        """
        Set rotation interpolator
        0: Ramp interpolator
        1: Streaming interpolator
        2: Velocity interpolator
        3: Trapezoidal interpolator
        """
        return self.__moby_stub.SetRotationInterpolator(IntVal(val=val))

    @Utils.exception_handler
    def set_drive_acc_dec(self, acc, dec):
        """
        Set drive acc dec
        """
        return self.__moby_stub.SetDriveAccDec(DoubleVals(val=[acc, dec]))

    @Utils.exception_handler
    def set_drive_interpolator_on_off(self, on):
        """
        Set drive interpolator On Off
        """
        return self.__moby_stub.SetDriveInterpolatorOnOff(BoolVal(val=on))

    @Utils.exception_handler
    def set_rotation_controller_type(self, val):
        """
        Set rotation controller type
        0: HINFPID_CONTROLLER,
        1: SIMPLEPID_POS_CONTROLLER,
        2: SIMPLEPID_VEL_CONTROLLER
        """
        return self.__moby_stub.SetRotationControllerType(IntVal(val=val))

    @Utils.exception_handler
    def set_rotation_gain(self, index, k, kv, kp):
        """
        Set Rotation Control Gain
        """
        return self.__moby_stub.SetControlParam(RotationGain(idx=index, k=k, kv=kv, kp=kp))

    @Utils.exception_handler
    def get_rotation_gain(self, index):
        """
        Get Rotation Control Gain (k, kv, kp)
        """
        val = self.__moby_stub.GetControlParam(IntVal(val=index))
        return {'k': val.k, 'kv':val.kv, 'kp':val.kp}
    
    @Utils.exception_handler
    def set_kinematics_forced(self, onoff):
        """
        Set Kinematics Forced( on=true, off=false )
        """
        self.__moby_stub.SetForceKinematics(BoolVal(val=onoff))

    @Utils.exception_handler
    def get_kinematics_forced(self):
        """
        Get Kinematics Forced( on=true, off=false )
        """
        val = self.__moby_stub.GetForceKinematics(Empty())
        return val

    ############################
    # Moby-Agri related commands
    ############################
        
    @Utils.exception_handler
    def turn_light(self, on):
        return self.__moby_stub.TurnLightOnOff(BoolVal(val=on))
        
    @Utils.exception_handler
    def turn_buzzer(self, on):
        return self.__moby_stub.TurnBuzzOnOff(BoolVal(val=on))

    @Utils.exception_handler
    def pause_bumper(self, on):
        return self.__moby_stub.PauseBumper(BoolVal(val=on))

    ############################
    # Moby Data logging
    ############################
    
    @Utils.exception_handler
    def start_rt_logging(self):
        """
        Start RT logging
        """
        return self.__moby_stub.StartRTLogging(Empty())
    
    @Utils.exception_handler
    def end_rt_logging(self):
        """
        End RT logging
        """
        return self.__moby_stub.EndRTLogging(Empty())




import json
import os
from pathlib import Path
import netifaces as ni

round_num = 3

tool_data_get = {}
program_tool_data = []
Convert_Modbus = True
Convert_Tool = True

class RobotProgramConverter:
    def __init__(self, input_data, dof):
        self.input_data = input_data
        self.float_val_num = 0
        self.bool_val_num = 0
        self.int_val_num = 0
        self.B_variaable = False
        self.operators = {0: "==", 1: "!=", 2: ">", 3: ">=", 4: "<", 5: "<="}
        self.init_data = input_data
        self.assignment_com  = 3
        self.set_DIO         = 4
        self.set_STOP        = 1
        self.loop_com        = 20
        self.loopBreak_com   = 21
        self.wait_com        = 22
        self.waitfor_com     = 23
        self.if_com          = 24
        self.elif_com        = 25
        self.else_com        = 26
        self.waitfor_di_com  = 28
        self.if_DI           = 29
        self.elif_DI         = 30
        self.set_tool        = 40
        self.home_com        = 100
        self.joint_absoulte  = 102
        self.task_absoulte   = 103
        self.joint_relative  = 104
        self.task_relative   = 105
        self.group           = 200
        self.pick            = 201
        self.place           = 202
        self.robot_dof       = dof

    def get_joint_A_motion(self, name, data,enable):
        for move in data["moveList"]:
            if move["name"] == name:
                move_vel = move["boundary"]["velLevel"]
                move_acc = move["boundary"]["accLevel"]
                shif_value = move["refFrame"]["tref"]
                move_data = {
                    "cmd": "MoveJ",
                    "name": name,
                    "wait_finish": True,
                    "base_type": "ABSOLUTE",
                    "interpolator_type": "VELOCITY",
                    "vel_ratio": move_vel*10,
                    "acc_ratio": move_acc*10,
                    "coll_sens_level": 3,
                    "blending": {"type": "NONE"},
                    "shift": [0]*self.robot_dof,
                    "shift_base": "RELATIVE",
                    "offsets": [[0]*self.robot_dof]*len(move["wpList"]),
                    "waypoint_time": [2, 2],
                    "waypoint_active": [],
                    "waypoints": [],
                    "active":enable
                }

                for waypoint in move["wpList"]:
                    try:
                        move_data["waypoint_active"].append(enable)
                        round_position = [round(position, round_num) for position in data["wpList"][waypoint["id"]]["q"]]
                        move_data["waypoints"].append(round_position)
                    except Exception as e:
                        print(e)
                return move_data

    def get_task_A_motion(self,name, data, enable):
        for move in data["moveList"]:
            if move["name"] == name:
                move_vel = move["boundary"]["velLevel"]
                move_acc = move["boundary"]["accLevel"]
                shif_value =[x * 1000 if i<3 else x for i,x in enumerate(move["refFrame"]["tref"])]
                move_data = {
                    "cmd": "MoveL",
                    "name": name,
                    "wait_finish": True,
                    "base_type": "ABSOLUTE",
                    "interpolator_type": "VELOCITY",
                    "vel_ratio": move_vel*10,
                    "acc_ratio": move_acc*10,
                    "coll_sens_level": 3,
                    "blending": {"type": "NONE"},
                    "shift": shif_value,
                    "shift_base": "RELATIVE",
                    "offsets": [[0]*self.robot_dof]*len(move["wpList"]),
                    "waypoint_time": [2, 2],
                    "waypoint_active": [],
                    "waypoints": [],
                    "waypoints_joint": [],
                    "active":enable
                }

                for waypoint in move["wpList"]:
                    try:
                        move_data["waypoint_active"].append(enable)
                        converting_point = [x * 1000 if i<3 else x for i,x in enumerate(data["wpList"][waypoint["id"]]["p"])]
                        round_position = [round(position, round_num) for position in converting_point]
                        round_joint_pos = [round(position, round_num) for position in data["wpList"][waypoint["id"]]["q"]]
                        move_data["waypoints"].append(round_position)
                        move_data["waypoints_joint"].append(round_joint_pos)
                    except Exception as e:
                        print(e)
                return move_data

    def get_joint_R_motion(self,name, data, enable):
        for move in data["moveList"]:
            if move["name"] == name:
                move_vel = move["boundary"]["velLevel"]
                move_acc = move["boundary"]["accLevel"]
                move_data = {
                    "cmd": "MoveJ",
                    "name": name,
                    "wait_finish": True,
                    "base_type": "RELATIVE",
                    "interpolator_type": "VELOCITY",
                    "vel_ratio": move_vel*10,
                    "acc_ratio": move_acc*10,
                    "coll_sens_level": 3,
                    "blending": {"type": "NONE"},
                    "shift": [0]*self.robot_dof,
                    "shift_base": "RELATIVE",
                    "offsets": [[0]*self.robot_dof]*len(move["wpList"]),
                    "waypoint_time": [2, 2],
                    "waypoint_active": [],
                    "waypoints": [],
                    "waypoints_origin": [],
                    "active":enable
                }

                for waypoint in move["wpList"]:
                    try:
                        move_data["waypoint_active"].append(enable)
                        round_position = [round(position, round_num) for position in data["wpList"][waypoint["id"]]["q"]]
                        round_origin = [round(position, round_num) for position in move["wpListOrigin"][0]["startPose"]["q"]]
                        move_data["waypoints"].append(round_position)
                        move_data["waypoints_origin"].append(round_origin)
                    except Exception as e:
                        print(e)
                return move_data

    def get_task_R_motion(self,name, data, enable):
        for move in data["moveList"]:
            if move["name"] == name:
                move_vel = move["boundary"]["velLevel"]
                move_acc = move["boundary"]["accLevel"]
                move_data = {
                    "cmd": "MoveL",
                    "name": name,
                    "wait_finish": True,
                    "base_type": "RELATIVE",
                    "interpolator_type": "VELOCITY",
                    "vel_ratio": move_vel*10,
                    "acc_ratio": move_acc*10,
                    "coll_sens_level": 3,
                    "blending": {"type": "NONE"},
                    "shift": [0,0,0,0,0,0],
                    "shift_base": "RELATIVE",
                    "offsets": [[0,0,0,0,0,0]]*len(move["wpList"]),
                    "waypoint_time": [2, 2],
                    "waypoint_active": [],
                    "waypoints": [],
                    "waypoints_origin": [],
                    "waypoints_joint": [],
                    "active":enable
                }

                for waypoint in move["wpList"]:
                    try:
                        converting_point = [x * 1000 if i<3 else x for i,x in enumerate(data["wpList"][waypoint["id"]]["p"])]
                        move_data["waypoints"].append(converting_point)
                        move_data["waypoint_active"].append(enable)
                        converting_origin = [x * 1000 if i<3 else x for i,x in enumerate(move["wpListOrigin"][0]["startPose"]["p"])]
                        move_data["waypoints_origin"].append(converting_origin)
                        move_data["waypoints_joint"].append(move["wpListOrigin"][0]["startPose"]["q"])
                        move_data["waypoints_joint"].append(move["wpListOrigin"][0]["endPose"]["q"])
                    except Exception as e:
                        print(e)
                return move_data

    def home_motion(self,data):
        move_data = {
            "cmd":"MoveHome",
            "vel_ratio":25,
            "acc_ratio":100,
            "active":data['enable']
        }
        return move_data

    def set_variable(self,data):
        set_data = []
        for i in range (len(data["varList"])):
            cur_name = data["varList"][i]["name"]
            if cur_name[0] == 'M':
                cur_name +='_WRITE'
            cur_value = data["varList"][i]["value"]
            set_data.append(str(cur_name)+" := "+str(cur_value))
        move_data = {
            "cmd":"SetVar",
            "active":data['enable'],
            "fold":False,
            "expression":set_data
        }
        return move_data

    def wait_command(self,data):
        cur_time = data["time"]
        move_data = {
            "cmd":"Sleep",
            "condition":{
                "type":"TIME",
                "time":cur_time
            },
            "async_sleep":False,
            "active":data['enable']
        }
        return move_data

    def loop_command(self,data):
        cur_count = data["count"]
        cur_id = data["id"]
        move_data = {
            "cmd":"Loop",
            "count":cur_count,
            "active":data['enable'],
            "fold":False,
            "children":[],
            "id":cur_id
        }
        return move_data

    def loop_break_command(self,data):
        move_data = {
            "cmd":"Break",
            "active":data['enable']
        }
        return move_data

    def wait_for_command(self,data):
        right_value = data["cond"]["right"]["value"]
        left_value = data["cond"]["left"]["value"]
        if left_value[0] == 'M':
            left_value +='_READ'
        op_value = data["cond"]["op"]
        expression = str(left_value)+self.operators[op_value]+str(right_value)
        move_data = {
            "cmd":"Sleep",
            "condition":{
                "type":"EXPRESSION",
                "expression":expression
            },
            "async_sleep":False,
            "active":data['enable']
        }
        return move_data

    def wait_for_di_command(self,data):
        condition = data["diList"]
        expresion = "("
        start = 0
        for cond in condition:
            ONOFF = "True"
            if start != 0:
                expresion+=(" and ")
            if cond["value"] == 0:
                ONOFF = "False"
            expresion+=("dinBoard["+str(cond["idx"])+"] == "+ ONOFF)
            start +=1
        expresion+=" )"
        ID = data["id"]

        move_data = {
            "cmd":"Sleep",
            "condition": {
                "type":"EXPRESSION",
                "expression":expresion,
            },
            "async_sleep":False,
            "active":data['enable']
        }
        return move_data

    def pick_place_command(self,data):
        work_type = "PICK"
        if data["type"] == 202:
            work_type = "PLACE"

        target_vel = data["target"]["boundary"]["velLevel"]
        target_acc = data["target"]["boundary"]["accLevel"]

        approach_vel = data["approach"]["boundary"]["velLevel"]
        approach_type = data["approach"]["direction"]
        approach_distance = data["approach"]["distance"]*1000

        retract_vel = data["approach"]["boundary"]["velLevel"]
        retract_type = data["retract"]["direction"]
        retract_distance = data["retract"]["distance"]*1000

        converting_point = [x * 1000 if i<3 else x for i,x in enumerate(data["target"]["point"]["p"])]
        target_pos_j = data["target"]["point"]["q"]
        move_data = {
            "cmd":"PickPlace",
            "work_type":work_type,
            "payload":0,
            "payload_set_tcp":False,
            "tar_pos":converting_point,
            "waypoints_joint":target_pos_j,
            "approach_pos":[0]*self.robot_dof,
            "retract_pos":[0]*self.robot_dof,
            "async_sleep":False,
            "vel_level":target_vel,
            "acc_level":target_acc,
            "vision_target":"None",
            "coll_sens_level":3,
            "approach_vel_level":approach_vel,
            "retract_vel_level":retract_vel,
            "approach_condition":{
                "type":"Constant",
                "value":True
            },
            "approach_check_cycle":0,
            "approach_wait_time":0,
            "retract_condition":{
                "type":"Constant",
                "value":True
            },
            "retract_check_cycle":0,
            "retract_wait_time":0,
            "active":data['enable'],
            "fold":False
        }
        move_data["approach_pos"][2]
        if approach_type <=1:
            if approach_type%2==1:
                move_data["approach_pos"][2] = -approach_distance
            else:
                move_data["approach_pos"][2] = approach_distance
        elif approach_type <=3:
            if approach_type%2==1:
                move_data["approach_pos"][0] = -approach_distance
            else:
                move_data["approach_pos"][0] = approach_distance
        elif approach_type <=5:
            if approach_type%2==1:
                move_data["approach_pos"][1] = -approach_distance
            else:
                move_data["approach_pos"][1] = approach_distance

        if retract_type <=1:
            if retract_type%2==1:
                move_data["retract_pos"][2] = -retract_distance
            else:
                move_data["retract_pos"][2] = retract_distance
        elif retract_type <=3:
            if retract_type%2==1:
                move_data["retract_pos"][0] = -retract_distance
            else:
                move_data["retract_pos"][0] = retract_distance
        elif retract_type <=5:
            if retract_type%2==1:
                move_data["retract_pos"][1] = -retract_distance
            else:
                move_data["retract_pos"][1] = retract_distance

        return move_data

    def if_command(self,data):
        right_value = data["cond"]["right"]["value"]
        left_value = data["cond"]["left"]["value"]
        if left_value[0] == 'M':
            left_value +='_READ'
        op_value = data["cond"]["op"]
        expression = str(left_value)+self.operators[op_value]+str(right_value)
        id = data["id"]
        move_data = {
            "cmd":"If",
            "condition":{
                "type":"Expression",
                "expression":expression
            },
            "active":data['enable'],
            "fold":False,
            "children":[],
            "id": id
        }
        return move_data

    def elif_command(self,data):
        right_value = data["cond"]["right"]["value"]
        left_value = data["cond"]["left"]["value"]
        if left_value[0] == 'M':
            left_value +='_READ'
        op_value = data["cond"]["op"]
        expression = str(left_value)+self.operators[op_value]+str(right_value)
        id = data["id"]
        move_data = {
            "cmd":"ElseIf",
            "condition":{
                "type":"Expression",
                "expression":expression
            },
            "active":data['enable'],
            "fold":False,
            "children":[],
            "id": id
        }
        return move_data

    def else_command(self,data):
        id = data["id"]
        move_data = {
            "cmd":"Else",
            "active":data['enable'],
            "fold":False,
            "children":[],
            "id": id
        }
        return move_data

    def if_dio_command(self,data):
        condition = data["diList"]
        expresion = "("
        start = 0
        for cond in condition:
            ONOFF = "True"
            if start != 0:
                expresion+=(" and ")
            if cond["value"] == 0:
                ONOFF = "False"
            expresion+=("dinBoard["+str(cond["idx"])+"] == "+ONOFF)
            start +=1
        expresion+=" )"
        ID = data["id"]
        move_data = {
            "cmd":"If",
            "condition": {
                "type":"Expression",
                "expression":expresion,
            },
            "active":data['enable'],
            "fold":False,
            "children":[],
            "id":ID
        }
        return move_data

    def elif_dio_command(self,data):
        condition = data["diList"]
        expresion = "("
        start = 0
        for cond in condition:
            ONOFF = "True"
            if start != 0:
                expresion+=(" and ")
            if cond["value"] == 0:
                ONOFF = "False"
            expresion+=("dinBoard["+str(cond["idx"])+"] == "+ ONOFF)
            start +=1
        expresion+=" )"
        ID = data["id"]
        move_data = {
            "cmd":"ElseIf",
            "condition": {
                "type":"Expression",
                "expression":expresion,
            },
            "active":data['enable'],
            "fold":False,
            "children":[],
            "id":ID
        }
        return move_data

    def group_command(self,data):
        name = data["groupName"]
        id = data["id"]
        move_data = {
            "cmd":"Group",
            "name": name,
            "active":data['enable'],
            "fold":False,
            "children":[],
            "id": id
        }
        return move_data

    def add_child_to_program(self,program_data, target_id, new_child):
        for item in program_data:
            if 'id' in item and item['id'] == target_id:
                item.setdefault('children', []).append(new_child)
            elif 'children' in item and item['children']:
                # Recursively check in the children
                updated_children = self.add_child_to_program(item['children'], target_id, new_child)
                if updated_children is not None:
                    item['children'] = updated_children
                    program_data
        return program_data

    def set_DIO_command(self,data) :
        do_list = data["doList"]
        converted_do_list = [{"address": item["idx"], "state": item["value"]} for item in do_list]
        move_data = {
            "cmd":"SetSignals",
            "do_signals":converted_do_list
        }
        return move_data
    def set_STOP_command(self,data) :
        move_data = {
            "cmd":"QuitProgram",
            "active":data['enable']
        }
        return move_data

    def set_Tool(self,data):
        try:            
            tool_num = str(data["toolCmd"]["toolId"])+"_"+str(data["toolCmd"]["cmdId"])
            tool_name = tool_data_get[tool_num]
        except:
            tool_name = "0_0"

        if tool_num in modbus_tool_ID:
            for dictionary in modbus_tool:
                if 'ID' in dictionary and dictionary['ID'] == tool_num:
                    break

            set_data = []
            for send_modbus in dictionary['value']:
                cur_name = send_modbus['name']
                cur_name +='_WRITE'
                cur_value = send_modbus["value"]
                set_data.append(str(cur_name)+" := "+str(cur_value))
            tool_data = {
                "cmd":"SetVar",
                "active":data['enable'],
                "fold":False,
                "expression":set_data
            }
        else:
            tool_data = {
                "cmd":"SetTool",
                "tool_name":tool_name,
                "active":data['enable']
            }
        return tool_data



    def convert_motion(self,robot_ip_address):
        input_data = self.init_data
        float_val_num = 0
        bool_val_num = 0
        int_val_num = 0
        B_variaable = False
        robot_ip = robot_ip_address
        output_data = {
            "Program": [
                {"cmd": "CollisionConfig", "policy": "PAUSE", "duration": 0},
                {"cmd": "VisionConfig"},
                {"cmd": "VariableConfig",
                 "local_list": [],
                 "modbus_list": [
                     {
                         "server_name":"my_robot_local",
                         "server_ip":"127.0.0.1",
                         "server_port":502,
                         "variable_list":[]
                     },
                     {
                         "server_name":"my_robot_ip",
                         "server_ip":robot_ip,
                         "server_port":502,
                         "variable_list":[]
                     }
                 ]
                 }
            ]
        }
        for val in input_data["program"][0]["toolInfo"]:
            program_tool_data.append(val)


        for val in input_data["program"][1]["varList"]: # 변수등록
            if val["name"][0] == 'B' and val["type"] == 5:
                B_variaable = True
                output_data["Program"][2]["local_list"].append(["INT",val["name"],int(''.join(filter(str.isdigit, val["name"]))),"LOCAL"])
            if val["name"][0] == 'M' and val["type"] == 5:
                output_data["Program"][2]["modbus_list"][1]["variable_list"].append(["WRITE_REGISTER",val["name"]+"_WRITE",int(''.join(filter(str.isdigit, val["name"]))),"LOCAL"])
                output_data["Program"][2]["modbus_list"][0]["variable_list"].append(["READ_REGISTER",val["name"]+"_READ",int(''.join(filter(str.isdigit, val["name"]))),"LOCAL"])
            if val["type"] == 2:  #float 변수
                output_data["Program"][2]["local_list"].append(["FLOAT",val["name"],float_val_num,"LOCAL"])
                float_val_num+=1
            if val["type"] == 3:  #boolean 변수
                output_data["Program"][2]["local_list"].append(["BOOL",val["name"],bool_val_num,"LOCAL"])
                bool_val_num+=1

        for val in input_data["program"][1]["varList"]: # 변수등록
            if B_variaable == False and val["type"] == 1:
                output_data["Program"][2]["local_list"].append(["INT",val["name"],int_val_num,"LOCAL"])
                int_val_num +=1

        for cur_program in input_data["program"]:
            if cur_program["type"] == 999:
                continue
            elif cur_program["type"] == 2:
                continue

            elif cur_program["type"] == self.assignment_com:
                return_data = self.set_variable(cur_program)

            elif cur_program["type"] == self.set_DIO:
                return_data = self.set_DIO_command(cur_program)

            elif cur_program["type"] == self.set_STOP:
                return_data = self.set_STOP_command(cur_program)

            elif cur_program["type"] == self.if_DI:
                return_data = self.if_dio_command(cur_program)
            elif cur_program["type"] == self.elif_DI:
                return_data = self.elif_dio_command(cur_program)
            elif cur_program["type"] == self.set_tool:
                return_data = self.set_Tool(cur_program)
            elif cur_program["type"] == self.waitfor_di_com:
                return_data = self.wait_for_di_command(cur_program)

            elif cur_program["type"] == self.loop_com:
                return_data = self.loop_command(cur_program)

            elif cur_program["type"] == self.loopBreak_com:
                return_data = self.loop_break_command(cur_program)

            elif cur_program["type"] == self.wait_com:
                return_data = self.wait_command(cur_program)
            elif cur_program["type"] == self.waitfor_com:
                return_data = self.wait_for_command(cur_program)
            elif cur_program["type"] == self.if_com:
                return_data = self.if_command(cur_program)

            elif cur_program["type"] == self.elif_com:
                return_data = self.elif_command(cur_program)
            elif cur_program["type"] == self.else_com:
                return_data = self.else_command(cur_program)

            elif cur_program["type"] == self.home_com:
                return_data = self.home_motion(cur_program)
            elif cur_program["type"] == self.joint_absoulte:
                name = cur_program["name"]
                return_data = self.get_joint_A_motion(name,input_data,cur_program['enable'])
            elif cur_program["type"] == self.task_absoulte:
                name = cur_program["name"]
                return_data = self.get_task_A_motion(name,input_data,cur_program['enable'])
            elif cur_program["type"] == self.joint_relative:
                name = cur_program["name"]
                return_data = self.get_joint_R_motion(name,input_data,cur_program['enable'])
            elif cur_program["type"] == self.task_relative:
                name = cur_program["name"]
                return_data = self.get_task_R_motion(name,input_data,cur_program['enable'])
            elif cur_program["type"] == self.group:
                return_data = self.group_command(cur_program)
            elif cur_program["type"] == self.pick:
                return_data = self.pick_place_command(cur_program)
            elif cur_program["type"] == self.place:
                return_data = self.pick_place_command(cur_program)

            else:
                return_data = [0,0,0,0]
            if cur_program["pId"] != 0:
                return_d =self.add_child_to_program(output_data["Program"],cur_program["pId"],return_data)
                output_data["Program"] = return_d
                continue
            else:
                output_data["Program"].append(return_data)
        return json.dumps(output_data, ensure_ascii=False)


if __name__ == "__main__":    
    current_working_directory = os.getcwd()+"/"

    files = [f for f in os.listdir('.') if os.path.isfile(f) and f.endswith('.json')]
    if not files:
        print("No JSON files found in the directory.")
    for i, file in enumerate(files):
        print(f"{i + 1}: {file}")
    file_index = int(input("Choose a file by entering its number: ")) - 1
    input_name = files[file_index]
    parts = input_name.split('.')
    if len(parts) < 3:
        print("File name format is incorrect.")
    before_robot_name = parts[-2]
    if before_robot_name in ["7","12","7v2","12v2"]:
        robot_dof = 6
    if before_robot_name in["rp2","rp2v2"]:
        if before_robot_name.startswith("rp"):
            before_robot_name  = before_robot_name.replace("rp", "RP", 1)
        robot_dof = 7
    robot_name = "indy"+before_robot_name    
    try:
        robot_ip = ni.ifaddresses('eth1')[ni.AF_INET][0]['addr']
    except:
        robot_ip = input("Enter the Robot IP Address: ")
    converting_name = input("Enter the output file name (without extension): ")    

    if Convert_Modbus:
        ipadress = robot_ip
        inputlocal = {'name': 'my_robot_local', 'ip': '127.0.0.1', 'port': 502}
        inputip = {'name': 'my_robot_ip', 'ip':ipadress, 'port': 502}
        try:
            with open("/home/user/release/IndyConfigurations/Cobot/Params/Modbus.json","r") as file:
                modbusdata = json.load(file)
            modbus_local = False
            modbus_ip = False

            for i in modbusdata['modbus_servers']:
                if i['name'] == 'my_robot_local':
                    modbus_local = True
                if i['name'] == 'my_robot_ip':
                    modbus_ip = True
                    if i['ip'] != ipadress:
                        i['ip']=ipadress
            if modbus_local == False:
                modbusdata['modbus_servers'].append(inputlocal)
            if modbus_ip == False:
                modbusdata['modbus_servers'].append(inputip)
            write_data = json.dumps(modbusdata)

            with open("/home/user/release/IndyConfigurations/Cobot/Params/Modbus.json", "w") as output_file:
                output_file.write(write_data)
        except:
            send_data = {"modbus_servers": []}
            send_data['modbus_servers'].append(inputlocal)
            send_data['modbus_servers'].append(inputip)
            write_data = json.dumps(send_data)
            with open("/home/user/release/IndyConfigurations/Cobot/Params/Modbus.json", "w") as output_file:
                output_file.write(write_data)

    if Convert_Tool:    
        with open(current_working_directory+input_name, "r") as file:
            input_data = json.load(file)

        for val in input_data["program"][0]["toolInfo"]:
            program_tool_data.append(val)
        convert_tool_data = {
            "tools": []
        }

        tool_idx = 0
        tool_subidx = 0


        modbus_tool=[]
        modbus_tool_ID=[]
        for tool_data in program_tool_data:
            for tool in tool_data['toolCommand']:
                append_data = {}
                tool_name = tool_data["name"] + "_" + tool["name"]
                tool_data_get[str(tool_idx)+"_"+str(tool_subidx)] = tool_name
                append_data["name"] = tool_name
                append_data["execute_time"] = tool["postwait"]
                append_data["do_signals"] = []
                append_data["enddo_signals"] = []
                append_data["ao_signals"] = []
                append_data["endao_signals"] = []
                append_data["expression"] = []

                if tool['commType'] == 4:
                    modbus_tool.append({"ID":str(tool_idx)+"_"+str(tool_subidx),"value": tool['varList']})
                    modbus_tool_ID.append(str(tool_idx)+"_"+str(tool_subidx))
                else:
                    if tool['commType'] == 1:
                        for do in tool["doMap"]:
                            append_data["do_signals"].append({"address": do["idx"], "state": do["value"]})

                    if tool['commType'] == 6:
                        tool_state = 2*(tool["endToolDO"]["toolType"]+1)*(tool["endToolDO"]["value"]-0.5)
                        append_data["enddo_signals"].append({"port":"C","states":[int(tool_state)]})
                    
                    convert_tool_data["tools"].append(append_data)
                    tool_subidx +=1
            tool_idx +=1
            tool_subidx = 0


        write_data = json.dumps(convert_tool_data)
        with open("/home/user/release/IndyConfigurations/Cobot/Configs/ToolList.json", "w") as output_file:
            output_file.write(write_data)


    with open(current_working_directory+input_name, "r") as file:
        input_data = json.load(file)

    converter = RobotProgramConverter(input_data, robot_dof)
    converted_program = converter.convert_motion(robot_ip)

    with open("/home/user/release/IndyDeployment/ProgramScripts/"+converting_name+"."+robot_name+".json", "w") as output_file:
        output_file.write(converted_program)


    print("converting_finish")


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: config.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import config_msgs_pb2 as config__msgs__pb2
import common_msgs_pb2 as common__msgs__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='config.proto',
  package='Nrmk.IndyFramework',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0c\x63onfig.proto\x12\x12Nrmk.IndyFramework\x1a\x11\x63onfig_msgs.proto\x1a\x11\x63ommon_msgs.proto2\x96\x1f\n\x06\x43onfig\x12\x45\n\x0bGetRefFrame\x12\x19.Nrmk.IndyFramework.Empty\x1a\x19.Nrmk.IndyFramework.Frame\"\x00\x12H\n\x0bSetRefFrame\x12\x19.Nrmk.IndyFramework.Frame\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12W\n\x11SetRefFramePlanar\x12\x1f.Nrmk.IndyFramework.PlanarFrame\x1a\x1f.Nrmk.IndyFramework.FrameResult\"\x00\x12I\n\x0cSetToolFrame\x12\x19.Nrmk.IndyFramework.Frame\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12J\n\rSetSpeedRatio\x12\x19.Nrmk.IndyFramework.Ratio\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12U\n\x10SetFTsensorFrame\x12!.Nrmk.IndyFramework.FTsensorFrame\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12U\n\x10GetFTsensorFrame\x12\x1c.Nrmk.IndyFramework.Response\x1a!.Nrmk.IndyFramework.FTsensorFrame\"\x00\x12S\n\x0fSetDIConfigList\x12 .Nrmk.IndyFramework.DIConfigList\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12P\n\x0fGetDIConfigList\x12\x19.Nrmk.IndyFramework.Empty\x1a .Nrmk.IndyFramework.DIConfigList\"\x00\x12S\n\x0fSetDOConfigList\x12 .Nrmk.IndyFramework.DOConfigList\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12P\n\x0fGetDOConfigList\x12\x19.Nrmk.IndyFramework.Empty\x1a .Nrmk.IndyFramework.DOConfigList\"\x00\x12O\n\x0fSetHomePosition\x12\x1c.Nrmk.IndyFramework.JointPos\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12L\n\x0fGetHomePosition\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1c.Nrmk.IndyFramework.JointPos\"\x00\x12Y\n\x0fSetAutoServoOff\x12&.Nrmk.IndyFramework.AutoServoOffConfig\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12V\n\x0fGetAutoServoOff\x12\x19.Nrmk.IndyFramework.Empty\x1a&.Nrmk.IndyFramework.AutoServoOffConfig\"\x00\x12W\n\x13SetJointControlGain\x12 .Nrmk.IndyFramework.JointGainSet\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12T\n\x13GetJointControlGain\x12\x19.Nrmk.IndyFramework.Empty\x1a .Nrmk.IndyFramework.JointGainSet\"\x00\x12U\n\x12SetTaskControlGain\x12\x1f.Nrmk.IndyFramework.TaskGainSet\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12R\n\x12GetTaskControlGain\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1f.Nrmk.IndyFramework.TaskGainSet\"\x00\x12_\n\x17SetImpedanceControlGain\x12$.Nrmk.IndyFramework.ImpedanceGainSet\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12\\\n\x17GetImpedanceControlGain\x12\x19.Nrmk.IndyFramework.Empty\x1a$.Nrmk.IndyFramework.ImpedanceGainSet\"\x00\x12W\n\x13SetForceControlGain\x12 .Nrmk.IndyFramework.ForceGainSet\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12T\n\x13GetForceControlGain\x12\x19.Nrmk.IndyFramework.Empty\x1a .Nrmk.IndyFramework.ForceGainSet\"\x00\x12U\n\x12SetTestControlGain\x12\x1f.Nrmk.IndyFramework.TestGainSet\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12R\n\x12GetTestControlGain\x12\x19.Nrmk.IndyFramework.Empty\x1a\x1f.Nrmk.IndyFramework.TestGainSet\"\x00\x12Y\n\x14SetCustomControlGain\x12!.Nrmk.IndyFramework.CustomGainSet\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12V\n\x14GetCustomControlGain\x12\x19.Nrmk.IndyFramework.Empty\x1a!.Nrmk.IndyFramework.CustomGainSet\"\x00\x12g\n\x19SetNewControllerTestOnOff\x12*.Nrmk.IndyFramework.NewControllerTestState\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12i\n\x1eGetNewControllerTestOnOffState\x12\x19.Nrmk.IndyFramework.Empty\x1a*.Nrmk.IndyFramework.NewControllerTestState\"\x00\x12V\n\x0fSetFrictionComp\x12#.Nrmk.IndyFramework.FrictionCompSet\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12S\n\x0fGetFrictionComp\x12\x19.Nrmk.IndyFramework.Empty\x1a#.Nrmk.IndyFramework.FrictionCompSet\"\x00\x12Q\n\x0bSetMountPos\x12\".Nrmk.IndyFramework.MountingAngles\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12N\n\x0bGetMountPos\x12\x19.Nrmk.IndyFramework.Empty\x1a\".Nrmk.IndyFramework.MountingAngles\"\x00\x12U\n\x0fSetToolProperty\x12\".Nrmk.IndyFramework.ToolProperties\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12R\n\x0fGetToolProperty\x12\x19.Nrmk.IndyFramework.Empty\x1a\".Nrmk.IndyFramework.ToolProperties\"\x00\x12Z\n\x10SetCollSensLevel\x12&.Nrmk.IndyFramework.CollisionSensLevel\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12W\n\x10GetCollSensLevel\x12\x19.Nrmk.IndyFramework.Empty\x1a&.Nrmk.IndyFramework.CollisionSensLevel\"\x00\x12[\n\x10SetCollSensParam\x12\'.Nrmk.IndyFramework.CollisionThresholds\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12X\n\x10GetCollSensParam\x12\x19.Nrmk.IndyFramework.Empty\x1a\'.Nrmk.IndyFramework.CollisionThresholds\"\x00\x12T\n\rSetCollPolicy\x12#.Nrmk.IndyFramework.CollisionPolicy\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12Q\n\rGetCollPolicy\x12\x19.Nrmk.IndyFramework.Empty\x1a#.Nrmk.IndyFramework.CollisionPolicy\"\x00\x12S\n\x0fSetSafetyLimits\x12 .Nrmk.IndyFramework.SafetyLimits\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12P\n\x0fGetSafetyLimits\x12\x19.Nrmk.IndyFramework.Empty\x1a .Nrmk.IndyFramework.SafetyLimits\"\x00\x12[\n\x13SetSafetyStopConfig\x12$.Nrmk.IndyFramework.SafetyStopConfig\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x12X\n\x13GetSafetyStopConfig\x12\x19.Nrmk.IndyFramework.Empty\x1a$.Nrmk.IndyFramework.SafetyStopConfig\"\x00\x12U\n\rSetCollTuning\x12$.Nrmk.IndyFramework.CollTuningConfig\x1a\x1c.Nrmk.IndyFramework.Response\"\x00\x62\x06proto3'
  ,
  dependencies=[config__msgs__pb2.DESCRIPTOR,common__msgs__pb2.DESCRIPTOR,])



_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_CONFIG = _descriptor.ServiceDescriptor(
  name='Config',
  full_name='Nrmk.IndyFramework.Config',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=75,
  serialized_end=4065,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetRefFrame',
    full_name='Nrmk.IndyFramework.Config.GetRefFrame',
    index=0,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._FRAME,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetRefFrame',
    full_name='Nrmk.IndyFramework.Config.SetRefFrame',
    index=1,
    containing_service=None,
    input_type=config__msgs__pb2._FRAME,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetRefFramePlanar',
    full_name='Nrmk.IndyFramework.Config.SetRefFramePlanar',
    index=2,
    containing_service=None,
    input_type=config__msgs__pb2._PLANARFRAME,
    output_type=config__msgs__pb2._FRAMERESULT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetToolFrame',
    full_name='Nrmk.IndyFramework.Config.SetToolFrame',
    index=3,
    containing_service=None,
    input_type=config__msgs__pb2._FRAME,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetSpeedRatio',
    full_name='Nrmk.IndyFramework.Config.SetSpeedRatio',
    index=4,
    containing_service=None,
    input_type=config__msgs__pb2._RATIO,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetFTsensorFrame',
    full_name='Nrmk.IndyFramework.Config.SetFTsensorFrame',
    index=5,
    containing_service=None,
    input_type=config__msgs__pb2._FTSENSORFRAME,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetFTsensorFrame',
    full_name='Nrmk.IndyFramework.Config.GetFTsensorFrame',
    index=6,
    containing_service=None,
    input_type=common__msgs__pb2._RESPONSE,
    output_type=config__msgs__pb2._FTSENSORFRAME,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetDIConfigList',
    full_name='Nrmk.IndyFramework.Config.SetDIConfigList',
    index=7,
    containing_service=None,
    input_type=config__msgs__pb2._DICONFIGLIST,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetDIConfigList',
    full_name='Nrmk.IndyFramework.Config.GetDIConfigList',
    index=8,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._DICONFIGLIST,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetDOConfigList',
    full_name='Nrmk.IndyFramework.Config.SetDOConfigList',
    index=9,
    containing_service=None,
    input_type=config__msgs__pb2._DOCONFIGLIST,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetDOConfigList',
    full_name='Nrmk.IndyFramework.Config.GetDOConfigList',
    index=10,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._DOCONFIGLIST,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetHomePosition',
    full_name='Nrmk.IndyFramework.Config.SetHomePosition',
    index=11,
    containing_service=None,
    input_type=config__msgs__pb2._JOINTPOS,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetHomePosition',
    full_name='Nrmk.IndyFramework.Config.GetHomePosition',
    index=12,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._JOINTPOS,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetAutoServoOff',
    full_name='Nrmk.IndyFramework.Config.SetAutoServoOff',
    index=13,
    containing_service=None,
    input_type=config__msgs__pb2._AUTOSERVOOFFCONFIG,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetAutoServoOff',
    full_name='Nrmk.IndyFramework.Config.GetAutoServoOff',
    index=14,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._AUTOSERVOOFFCONFIG,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetJointControlGain',
    full_name='Nrmk.IndyFramework.Config.SetJointControlGain',
    index=15,
    containing_service=None,
    input_type=config__msgs__pb2._JOINTGAINSET,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetJointControlGain',
    full_name='Nrmk.IndyFramework.Config.GetJointControlGain',
    index=16,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._JOINTGAINSET,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetTaskControlGain',
    full_name='Nrmk.IndyFramework.Config.SetTaskControlGain',
    index=17,
    containing_service=None,
    input_type=config__msgs__pb2._TASKGAINSET,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetTaskControlGain',
    full_name='Nrmk.IndyFramework.Config.GetTaskControlGain',
    index=18,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._TASKGAINSET,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetImpedanceControlGain',
    full_name='Nrmk.IndyFramework.Config.SetImpedanceControlGain',
    index=19,
    containing_service=None,
    input_type=config__msgs__pb2._IMPEDANCEGAINSET,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetImpedanceControlGain',
    full_name='Nrmk.IndyFramework.Config.GetImpedanceControlGain',
    index=20,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._IMPEDANCEGAINSET,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetForceControlGain',
    full_name='Nrmk.IndyFramework.Config.SetForceControlGain',
    index=21,
    containing_service=None,
    input_type=config__msgs__pb2._FORCEGAINSET,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetForceControlGain',
    full_name='Nrmk.IndyFramework.Config.GetForceControlGain',
    index=22,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._FORCEGAINSET,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetTestControlGain',
    full_name='Nrmk.IndyFramework.Config.SetTestControlGain',
    index=23,
    containing_service=None,
    input_type=config__msgs__pb2._TESTGAINSET,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetTestControlGain',
    full_name='Nrmk.IndyFramework.Config.GetTestControlGain',
    index=24,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._TESTGAINSET,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetCustomControlGain',
    full_name='Nrmk.IndyFramework.Config.SetCustomControlGain',
    index=25,
    containing_service=None,
    input_type=config__msgs__pb2._CUSTOMGAINSET,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetCustomControlGain',
    full_name='Nrmk.IndyFramework.Config.GetCustomControlGain',
    index=26,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._CUSTOMGAINSET,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetNewControllerTestOnOff',
    full_name='Nrmk.IndyFramework.Config.SetNewControllerTestOnOff',
    index=27,
    containing_service=None,
    input_type=config__msgs__pb2._NEWCONTROLLERTESTSTATE,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetNewControllerTestOnOffState',
    full_name='Nrmk.IndyFramework.Config.GetNewControllerTestOnOffState',
    index=28,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._NEWCONTROLLERTESTSTATE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetFrictionComp',
    full_name='Nrmk.IndyFramework.Config.SetFrictionComp',
    index=29,
    containing_service=None,
    input_type=config__msgs__pb2._FRICTIONCOMPSET,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetFrictionComp',
    full_name='Nrmk.IndyFramework.Config.GetFrictionComp',
    index=30,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._FRICTIONCOMPSET,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetMountPos',
    full_name='Nrmk.IndyFramework.Config.SetMountPos',
    index=31,
    containing_service=None,
    input_type=config__msgs__pb2._MOUNTINGANGLES,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetMountPos',
    full_name='Nrmk.IndyFramework.Config.GetMountPos',
    index=32,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._MOUNTINGANGLES,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetToolProperty',
    full_name='Nrmk.IndyFramework.Config.SetToolProperty',
    index=33,
    containing_service=None,
    input_type=config__msgs__pb2._TOOLPROPERTIES,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetToolProperty',
    full_name='Nrmk.IndyFramework.Config.GetToolProperty',
    index=34,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._TOOLPROPERTIES,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetCollSensLevel',
    full_name='Nrmk.IndyFramework.Config.SetCollSensLevel',
    index=35,
    containing_service=None,
    input_type=config__msgs__pb2._COLLISIONSENSLEVEL,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetCollSensLevel',
    full_name='Nrmk.IndyFramework.Config.GetCollSensLevel',
    index=36,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._COLLISIONSENSLEVEL,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetCollSensParam',
    full_name='Nrmk.IndyFramework.Config.SetCollSensParam',
    index=37,
    containing_service=None,
    input_type=config__msgs__pb2._COLLISIONTHRESHOLDS,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetCollSensParam',
    full_name='Nrmk.IndyFramework.Config.GetCollSensParam',
    index=38,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._COLLISIONTHRESHOLDS,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetCollPolicy',
    full_name='Nrmk.IndyFramework.Config.SetCollPolicy',
    index=39,
    containing_service=None,
    input_type=config__msgs__pb2._COLLISIONPOLICY,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetCollPolicy',
    full_name='Nrmk.IndyFramework.Config.GetCollPolicy',
    index=40,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._COLLISIONPOLICY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetSafetyLimits',
    full_name='Nrmk.IndyFramework.Config.SetSafetyLimits',
    index=41,
    containing_service=None,
    input_type=config__msgs__pb2._SAFETYLIMITS,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetSafetyLimits',
    full_name='Nrmk.IndyFramework.Config.GetSafetyLimits',
    index=42,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._SAFETYLIMITS,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetSafetyStopConfig',
    full_name='Nrmk.IndyFramework.Config.SetSafetyStopConfig',
    index=43,
    containing_service=None,
    input_type=config__msgs__pb2._SAFETYSTOPCONFIG,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetSafetyStopConfig',
    full_name='Nrmk.IndyFramework.Config.GetSafetyStopConfig',
    index=44,
    containing_service=None,
    input_type=common__msgs__pb2._EMPTY,
    output_type=config__msgs__pb2._SAFETYSTOPCONFIG,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetCollTuning',
    full_name='Nrmk.IndyFramework.Config.SetCollTuning',
    index=45,
    containing_service=None,
    input_type=config__msgs__pb2._COLLTUNINGCONFIG,
    output_type=common__msgs__pb2._RESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_CONFIG)

DESCRIPTOR.services_by_name['Config'] = _CONFIG

# @@protoc_insertion_point(module_scope)



import numpy as np
from math import *
import os
import json
import socket
import netifaces
from dataclasses import dataclass, field
from .jsmin import jsmin

import grpc
from neuromeka.enums import Limits


@dataclass
class GRPCReturn:
    code: grpc.StatusCode = grpc.StatusCode.OK
    details: str = ''


def exception_handler(func):
    def wrapper(self, *args, **kwargs):
        try:
            return func(self, *args, **kwargs)
        except grpc.RpcError as ex:
            # print('Function: ' + str(func.__name__) + ' received gRPC error: ' + str(ex))
            print('GRPC Exception at ' + str(func.__name__) + ' (' + str(ex.code()) + ' - ' + str(ex.details()) + ')')
            return None

    return wrapper


def exception_forwarder(func):
    def wrapper(self, *args, **kwargs):
        try:
            return func(self, *args, **kwargs)
        except grpc.RpcError as ex:
            # print('Function: ' + str(func.__name__) + ' received gRPC error: ' + str(ex))
            print('GRPC Exception Forwarder at ' + str(func.__name__) + ' (' + str(ex.code()) + ' - ' + str(
                ex.details()) + ')')
            return GRPCReturn(code=ex.code(), details=ex.details())

    return wrapper


def get_ip():
    ip_addr = ''
    interfaces = netifaces.interfaces()
    for interface in interfaces:
        if interface != 'lo':
            ip = netifaces.ifaddresses(interface)
            if netifaces.AF_INET in ip:
                ip_addr = ip[netifaces.AF_INET][0]['addr']
    return ip_addr


def get_all_ip():
    ip_list = []
    interfaces = netifaces.interfaces()
    for interface in interfaces:
        ip = netifaces.ifaddresses(interface)
        if netifaces.AF_INET in ip:
            ip_list.append(ip[netifaces.AF_INET][0]['addr'])
    return ip_list


def get_middleware_path():
    __common_dir = os.path.dirname(os.path.abspath(__file__))
    __middleware_dir = os.path.dirname(__common_dir)
    return __middleware_dir


def get_deploy_path():
    __deploy_dir = os.path.dirname(get_middleware_path())
    return __deploy_dir


##
# @brief convert to absolute path if path is relative (not start with '/')
def get_abs_path(path: str):
    if path.startswith("/"):
        return path
    else:
        return os.path.join(get_deploy_path(), path)


def write_json(file_name_abs, dict_obj):
    with open(file_name_abs, 'w') as f:
        json.dump(dict_obj, f, indent=4)


def load_json(file_name_abs):
    def dupe_checking_hook(pairs):
        result = dict()
        for key, val in pairs:
            if key in result:
                raise KeyError("Duplicate key specified: %s" % key)
            result[key] = val
        return result

    if os.path.isfile(file_name_abs):
        with open(file_name_abs) as f:
            decoder = json.JSONDecoder(object_pairs_hook=dupe_checking_hook)
            json_min_raw = jsmin(f.read())
            data_dic = decoder.decode(json_min_raw)
            return data_dic
    else:
        return None


def rot_axis(axis, degree):
    th = radians(degree)
    if axis == 1:
        rot_matrix = np.asarray([[1, 0, 0], [0, cos(th), -sin(th)], [0, sin(th), cos(th)]])
    elif axis == 2:
        rot_matrix = np.asarray([[cos(th), 0, sin(th)], [0, 1, 0], [-sin(th), 0, cos(th)]])
    elif axis == 3:
        rot_matrix = np.asarray([[cos(th), -sin(th), 0], [sin(th), cos(th), 0], [0, 0, 1]])
    else:
        rot_matrix = np.identity
    return rot_matrix


def euler_to_rotm(uvw):
    rx = uvw[0]
    ry = uvw[1]
    rz = uvw[2]
    return np.matmul(np.matmul(rot_axis(3, rz), rot_axis(2, ry)), rot_axis(1, rx))


def rotm_to_euler(rot_matrix):
    sy = sqrt(rot_matrix[0, 0] ** 2 + rot_matrix[1, 0] ** 2)

    if sy > 0.000001:
        u = degrees(atan2(rot_matrix[2, 1], rot_matrix[2, 2]))
        v = degrees(atan2(-rot_matrix[2, 0], sy))
        w = degrees(atan2(rot_matrix[1, 0], rot_matrix[0, 0]))
    else:
        u = degrees(atan2(-rot_matrix[1, 2], rot_matrix[1, 1]))
        v = degrees(atan2(-rot_matrix[2, 0], sy))
        w = 0

    return np.asarray([u, v, w])


def pos_to_transform(p):
    xyz = p[:3]
    uvw = p[3:]
    transform_matrix = np.identity(4)
    transform_matrix[:3, :3] = euler_to_rotm(uvw)
    transform_matrix[:3, 3] = xyz[:]
    return transform_matrix


def transform_to_pos(transform_matrix):
    p = [0, 0, 0, 0, 0, 0]
    p[:3] = transform_matrix[:3, 3]
    p[3:] = list(rotm_to_euler(transform_matrix[:3, :3]))
    return p


def to_vel_ratio(level):
    """
    Params:
        | level: 1 ~ 9
    Return:
        | joint_vel = level x 10 deg
    """
    if level < Limits.LevelMin:
        level = Limits.LevelMin
    if level > Limits.LevelMax:
        level = Limits.LevelMax

    if level > 2:
        vel_ratio = Limits.VelAutoLevelValue * (level - 1)
    else:
        vel_ratio = Limits.JogVelRatioMin + Limits.VelManualLevelValue * (level - 1)

    # joint_vel = level * 10
    return vel_ratio


def to_acc_ratio(level):
    """
    Params:
        | level: 1 ~ 9
        | joint_vel: deg
    Return:
        | joint_acc = level x joint_vel
    """
    if level < Limits.LevelMin:
        level = Limits.LevelMin
    if level > Limits.LevelMax:
        level = Limits.LevelMax

    acc_ratio = Limits.JogAccRatioDefault * level

    return acc_ratio

# def to_task_vel(level):
#     """
#     Params:
#         | level: 1 ~ 9
#     Return:
#         | disp_vel = level x 25 mm
#         | rot_vel = level x 10 deg
#     """
#     if level < 1:
#         level = 1
#     if level > 9:
#         level = 9
#
#     disp_vel = common.VelTaskDispMin + common.VelTaskDispDiff*(level-1)
#     rot_vel = common.VelTaskRotMin + common.VelTaskRotDiff*(level-1)
#
#     # disp_vel = level * 25
#     # rot_vel = level * 10
#
#     return disp_vel, rot_vel


# def to_task_acc(level, disp_vel, rot_vel):
#     """
#     Params:
#         | level: 1 ~ 9
#         | disp_vel: mm
#         | rot_vel: deg
#     Return:
#         | disp_acc = level x disp_vel
#         | rot_acc = level x rot_vel
#     """
#     if level < 1:
#         level = 1
#     if level > 9:
#         level = 9
#
#     disp_acc = level * disp_vel
#     rot_acc = level * rot_vel
#     return disp_acc, rot_acc



import os
import sys

generated_files_path = os.path.abspath(os.path.dirname(__file__))
sys.path.append(generated_files_path)

path_to_remove1 = "/home/user/dev/runtest/Release/IndyDeployment/PythonMiddleware/interfaces/impl"
path_to_remove2 = "/home/user/release/IndyDeployment/PythonMiddleware/interfaces/impl"
if path_to_remove1 in sys.path: sys.path.remove(path_to_remove1)
if path_to_remove2 in sys.path: sys.path.remove(path_to_remove2)

from .ethercat_pb2_grpc import *
from .control_pb2_grpc import *
from .device_pb2_grpc import *
from .config_pb2_grpc import *
from .rtde_pb2_grpc import *
from .moby_pb2_grpc import *

from . import ethercat_msgs_pb2 as ethercat_msgs
from . import common_msgs_pb2 as common_msgs
from . import control_msgs_pb2 as control_msgs
from . import config_msgs_pb2 as config_msgs
from . import device_msgs_pb2 as device_msgs
from . import rtde_msgs_pb2 as rtde_msgs
from . import moby_msgs_pb2 as moby_msgs

